'use strict';

var deepmerge = require('deepmerge');
var changeCase = require('change-case');

module.exports = function (swagger, options) {
  return mergeConfigs(swaggerToDefinitions(swagger, options).filter(function (definition) {
    return isTarget(definition, options);
  }).map(function (definition) {
    return definitionToConfig(definition, options);
  }));
};

var swaggerToDefinitions = function swaggerToDefinitions(swaggers, options) {
  var definitions = [];

  swaggers.forEach(function (swagger) {
    swagger.paths.getItems().forEach(function (pathItem) {
      var path = pathItem.path();
      var methodNames = Object.keys(pathItem).filter(function (k) {
        return !k.startsWith("_");
      });
      methodNames.forEach(function (method) {
        definitions.push({
          path: path,
          method: method.toLowerCase(),
          methodObject: pathItem[method]
        });
      });

      if (options.optionsMethod) {
        var filtered = methodNames.filter(function (method) {
          return method.toLowerCase() !== 'get';
        });
        if (filtered.length > 0) {
          definitions.push({
            path: path,
            method: 'options',
            methodObject: pathItem[filtered[0]]
          });
        }
      }
    });
  });

  return definitions;
};

var isTarget = function isTarget(definition, options) {
  if (typeof definition.methodObject.tags === 'undefined') {
    return false;
  }

  return definition.methodObject.tags.some(function (tag) {
    return tag.indexOf(options.apiPrefix) === 0;
  });
};

var definitionToConfig = function definitionToConfig(definition, options) {
  var service = extractServiceName(definition, options);

  var functionName = void 0;
  if (options.functionName) {
    functionName = options.functionName;
  } else {
    functionName = extractFunctionName(definition, options);
  }

  var handler = 'handler.' + functionName;

  var path = void 0;
  if (options.basePath) {
    var splited = definition.path.slice(1).split('/');
    path = splited.length === 1 ? '/' : '/' + splited.slice(1).join('/');
  } else {
    path = definition.path;
  }

  var httpEvent = {
    http: {
      path: path,
      method: definition.method,
      integration: 'lambda-proxy'
    }
  };

  var params = extractParameters(definition);

  if (params != null) {
    httpEvent.http['request'] = {
      parameters: params
    };
  }

  if (options.cors || options.optionsMethod && definition.method === 'get') {
    httpEvent.http['cors'] = true;
  }

  if (options.authorizer) {
    httpEvent.http['authorizer'] = options.authorizer;
  }

  var events = [httpEvent];
  var functions = {};
  functions[functionName] = { handler: handler, events: events };

  return { service: service, functions: functions };
};

var extractServiceName = function extractServiceName(definition, options) {
  var extraced = definition.methodObject.tags.filter(function (tag) {
    return tag.indexOf(options.apiPrefix) === 0;
  })[0];
  var caseChanged = changeCase.paramCase(extraced.slice(options.apiPrefix.length + 1));

  return options.servicePrefix ? options.servicePrefix + '-' + caseChanged : caseChanged;
};

var extractFunctionName = function extractFunctionName(definition, options) {
  var method = [definition.method];

  if (options.operationId && definition.methodObject && typeof definition.methodObject.operationId === 'string') {
    return definition.methodObject.operationId;
  }

  var resources = definition.path.split('/').filter(function (w) {
    return w.length > 0;
  }).filter(function (w, i) {
    return options.basePath ? i !== 0 : true;
  }).reduce(function (acc, current, index, arr) {
    if (/^\{.*\}$/.test(current)) {
      return acc;
    } else {
      if (/^\{.*\}$/.test(arr[index - 1])) {
        return [current];
      } else {
        return acc.concat(current);
      }
    }
  }, []).filter(function (w) {
    return !/^\{.*\}$/.test(w);
  }).map(function (w) {
    return changeCase.pascalCase(w);
  });

  var conditions = definition.path.split('/').filter(function (w) {
    return w.length > 0;
  }).filter(function (w) {
    return (/^\{.*\}$/.test(w)
    );
  }).map(function (w, i) {
    var n = changeCase.dotCase(w.slice(1, -1)).split('.').map(function (s) {
      return changeCase.pascalCase(s.slice(0, 3));
    }).join('');
    return i === 0 ? 'With' + n : n;
  });

  return method.concat(resources, conditions).join('');
};

var extractParameters = function extractParameters(definition) {

  // Build the serverless parameters object
  var serverlessParameters = {
    paths: {}
  };

  var nbParams = 0;

  var swaggerParameters = definition.methodObject.parameters;

  if (swaggerParameters) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = swaggerParameters[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var param = _step.value;

        if (param.in === 'path' && param.required) {
          nbParams++;
          serverlessParameters.paths[param.name] = true;
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  }

  if (nbParams === 0) {
    return null;
  }

  return serverlessParameters;
};

var mergeConfigs = function mergeConfigs(configs) {
  var nameSet = new Set();
  configs.forEach(function (config) {
    return nameSet.add(config.service);
  });

  return Array.from(nameSet).map(function (name) {
    var merged = { service: name, functions: {} };

    configs.forEach(function (config) {
      if (name === config.service) {
        merged.functions = deepmerge(merged.functions, config.functions);
      }
    });

    return merged;
  });
};
module.exports._extractFunctionName = extractFunctionName;
module.exports._definitionToConfig = definitionToConfig;